#!/usr/bin/python
#==============================================================================
#== Title: dwPY_OpenMQTT.py
#== Description:  Simple MQTT/JSON Python Edge Device Client (Python - Paho MQTT Library)
#==
#== Copyright (c)2015 ILS Technology, LLC.
#== http://www.ilstechnology.com
#== http://www.devicewise.com
#==
#== To Execute:
#== python dwPY_OpenMQTT.py
#==
#== Legal Disclaimer:
#==
#== THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#== INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#== PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#== HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#== OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#== SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#==============================================================================

# Load Python Import Libraries
import sys
import time
import threading
import dwopenmqtt
import grovepi


# dwM2M Developer Community Assigned Tokens...
dwApiURL    = "http://api.devicewise.com/api";
dwApiHost   = "api.devicewise.com";

# dwOpen Member/Thing Assigned Tokens...
dwAppToken  = "????????????????";
dwThingKey  = "PythonRaspberryPiThing";

# =============================================================================
# == Function: userCallBack
# == Purpose:  Handles Method Callbacks
# =============================================================================
def userCallBack( MsgID, MethodName, ParamText ):
  
	print "*** userCallBack... method = ", MethodName
	print "*** ParamText = ", ParamText	
	
	if( MethodName == "setled" ):
		ledValue = dwopen.getFramedText(ParamText, '\"varled\":', '}');
		print "*** Var LED = ", ledValue;
		
		if ledValue == "true":
			print "*** LED ON!!!"
			grovepi.digitalWrite(led,1)		# turn ON LED
		else:
			print "*** LED OFF!!!"
			grovepi.digitalWrite(led,0)		# turn OFF LED
			
		dwopen.dwMailboxAck( MsgID, 0, '', '' );		
		
	if( MethodName == "beep" ):
		mSDuration = int(dwopen.getFramedText(ParamText, '\"duration\":', '}'));
		print "*** Beep Duration = ", mSDuration;
		
		if (mSDuration > 2000) or (mSDuration < 0):
			print "*** Invalid Duration Value!!!"
			dwopen.dwMailboxAck( MsgID, -1, 'Invalid Beep Duration - must be within 0 and 2000', '' );	
			
		else:
			print "*** Beep Duration =", mSDuration	

			grovepi.digitalWrite(buzzer,1);			
			time.sleep( mSDuration/1000.0 );
			grovepi.digitalWrite(buzzer,0);			
			
			dwopen.dwMailboxAck( MsgID, 0, '', '' );	

	return;
	

# =============================================================================
# == Setup I/O
# =============================================================================

# Connect the Grove Rotary Angle Sensor to analog port A0
potentiometer = 0
grovepi.pinMode(potentiometer,"INPUT")

# Connect the Grove Rotary Angle Sensor to analog port A1
tempsensor = 1
grovepi.pinMode(tempsensor,"INPUT")

# Connect the Grove LED to digital port D4
led = 4
grovepi.pinMode(led,"OUTPUT")

# Connect the Grove Buzzer to digital port D8
buzzer = 8
grovepi.pinMode(buzzer,"OUTPUT")

# Reference voltage of ADC is 5v
adc_ref = 5

# Vcc of the grove interface is normally 5v
grove_vcc = 5

# Full value of the rotary angle is 300 degrees, as per it's specs (0 to 300)
full_angle = 300

# =============================================================================
# == Main Program Logic
# =============================================================================

# Display Banner Information...
print "deviceWISE OpenAPI MQTT/JSON Client Example Program (Python - Paho MQTT Library Base)"
print "(c) ILS Technology, 2014.  All Rights Reserved.\n"

print("Load dwOpenMQTT Library...")
dwopen = dwopenmqtt.dwOpen();
print("dwOpenMQTT Library Loaded...")

#------------------------------------------------
#-- Connect to m2mAIR Cloud Server
#------------------------------------------------
print("Connect to m2mAIR Cloud Server...")
rc = dwopen.mqttConnect( dwApiHost, dwThingKey, dwAppToken, userCallBack );
print "dwOpen API - MQTT Connect Returned... rc = ", rc

print("Waiting 3 Seconds...")
time.sleep( 3 )

print("Begin Publishing Requests to m2mAIR Cloud Server...")

#------------------------------------------------
#-- Log Publish
#------------------------------------------------
print "dwOpen API - Executing Log Publish Request..."
msgText  = "Python MQTT Edge Device... Connected and Active!"

rc = dwopen.dwLogPublish( dwThingKey, msgText );
print "dwOpen API - Log Publish Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Location Publish
#------------------------------------------------
print "dwOpen API - Sending Location Publish Request..."
#ILST Headquarters - GPS Detail...
locLatitude  = "26.394919"
locLongitude = "-80.112808"
locAltitude  = "11.4"
locSpeed     = "0.0"
locHeading   = "18.8"
locFixType   = "manual"

rc = dwopen.dwLocationPublish( dwThingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading, locFixType );
print "dwOpen API - Location Publish Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Set Attribute
#------------------------------------------------
print "dwOpen API - Executing Set Attribute Request..."

attribKey   = "SerialNumber"
attribValue = "0123456789"

rc = dwopen.dwSetAttribute( dwThingKey, attribKey, attribValue );
print "dwOpen API - Set Attribute Returned... rc = ", rc

time.sleep( 2 )

#------------------------------------------------
#-- Property Publish
#------------------------------------------------

LastDegrees = -1

for i in range(0,100):

	# Get temp from the Grove temp sensor...
	# 	The second argument in the grovepi.temp() method defines which board version you have connected.
	# 		temp = grovepi.temp(sensor)        # B value = 3975
	# 		temp = grovepi.temp(sensor,'1.1')  # B value = 4250
	# 		temp = grovepi.temp(sensor,'1.2')  # B value = 4250
	temp = grovepi.temp(tempsensor,'1.1');
	print "temp =", temp
	
	# Publish temp property data
	print "dwOpen API - Temp Property Publish..."	
	rc = dwopen.dwPropertyPublish( dwThingKey, "Temp", temp )
		
	# Read sensor value from potentiometer
	sensor_value = grovepi.analogRead(potentiometer);

	# Calculate voltage
	voltage = round((float)(sensor_value) * adc_ref / 1023, 2)

	# Calculate rotation in degrees (0 to 300)
	degrees = round((voltage * full_angle) / grove_vcc, 2)
	
	# Did the temp change?
	if degrees <> LastDegrees:
		# Yes, Publish new rotary value
		print "dwOpen API - Executing Rotary Property Publish..."
		rc = dwopen.dwPropertyPublish( dwThingKey, "Rotary", degrees )			
		print "dwOpen API - Rotary Property Publish Returned... rc = ", rc

		print ("sensor_value =", sensor_value, " voltage =", voltage, " degrees =", degrees)

		#------------------------------------------------
		#-- Alarm Publish
		#------------------------------------------------

		# Various simulated alarm status...
		if degrees == 0:
			alarmState = 0
			alarmMsg   = "Temp - OK"
			print "dwOpen API - Executing Alarm Publish Request... OK"		
		else:
			if degrees < 200:
				alarmState = 1
				alarmMsg   = "Temp - Warning"		
				print "dwOpen API - Executing Alarm Publish Request... Warning"
			else:
				alarmState = 2
				alarmMsg   = "Temp - HIGH"			
				print "dwOpen API - Executing Alarm Publish Request... HIGH"
					
		rc = dwopen.dwAlarmPublish( dwThingKey, "alarm1", alarmState, alarmMsg );
		print "dwOpen API - Alarm Publish Returned... rc = ", rc

	# Save our last rotary reading	
	LastDegrees = degrees

	#------------------------------------------------
	#-- Await Incoming Method Execute Requests
	#------------------------------------------------
	
	print "dwOpen API - Awaiting Method Execute Requests..."

	# Wait here for about 10 seconds for cloud methods before proceeding
	for i in range(0,10):
		time.sleep( 1 );
		dwopen.dwCheck();

#------------------------------------------------
#-- Disconnect from m2mAIR Cloud Server
#------------------------------------------------
print("Disconnecting from m2mAIR Cloud Server...")
dwopen.mqttDisconnect()
print("Ok...")

# Exiting the Application...
print( "dwOpen API MQTT/JSON Example Processing Complete, Exiting... Good Bye!" );
sys.exit()


